@@ -6,7 +6,7 @@ import dash_bootstrap_components as dbc
 import dash_core_components as dcc
 import dash_html_components as html
 import numpy as np
-from dash.dependencies import Input, Output, State, ALL
+from dash.dependencies import Input, Output, State, ALL, MATCH
 from dash_cool_components import TagInput, DateTimePicker
 from pathlib import Path
 from collections import Counter
@@ -152,10 +152,10 @@ class SchemaFormItem(dbc.FormGroup):
 
         elif value['type'] == 'boolean':
             compound_id['data_type'] = 'boolean'
-            default = value['default']
+            default = value.get('default', False)
             field_input = dbc.Checkbox(
                 id=compound_id,
-                checked=value['default']
+                checked=default
             )
 
         else:
@@ -254,6 +254,7 @@ class SchemaForm(dbc.Card):
         self.schema = schema
         self.owner_class = schema.get('tag', '')
         self.parent_form = parent_form
+        self.skiped_forms = []
 
         # Unique Card IDs are composed by parent id + key from json schema
         if parent_form is None:
@@ -267,11 +268,10 @@ class SchemaForm(dbc.Card):
 
         header_text = schema.get('title', self.id.split('-')[-1])
         self.header = dbc.CardHeader(
-            [html.H4(header_text, className="title_" + key)],
+            [dbc.Button(html.H4(header_text, style={"color": 'black'}, className="title_" + key), color='link', id={"type": "collapsible-toggle", "container": f"{self.container.id}", 'index': f'{self.id}-collapsible'})],
             style={'padding': '10px'}
         )
         self.body = dbc.CardBody([])
-        self.children = [self.header, self.body]
 
         self.required_fields = schema.get('required', '')
 
@@ -281,11 +281,19 @@ class SchemaForm(dbc.Card):
         if 'properties' in schema:
             self.make_form(properties=schema['properties'])
 
+        self.body = dbc.Collapse(self.body, id={"type": 'collapsible-body', "container": f"{self.container.id}" ,"index": f'{self.id}-collapsible'}, is_open=True)
+        self.children = [self.header, self.body]
+
+
     def make_form(self, properties):
         """Iterates over properties of schema and assembles form items"""
         for k, v in properties.items():
             required = k in self.required_fields
 
+            if 'renderForm' in v and not v['renderForm']:
+                self.skiped_forms.append(k)
+                continue
+
             # If item is an object or reference to an object on definitions, make subform
             if 'type' in v and v['type'] == 'object':
                 item = SchemaForm(schema=v, key=k, parent_form=self)
@@ -301,13 +309,17 @@ class SchemaForm(dbc.Card):
             # If item is a field
             if 'type' in v and (v['type'] == 'array'):
                 # If field is an array of subforms, e.g. ImagingPlane.optical_channels
-                if isinstance(v['items'], list):
+                if 'minItems' in v: 
                     value = []
-                    if '$ref' in v['items'][0]:
-                        template_name = v['items'][0]['$ref'].split('/')[-1]
-                        schema = self.definitions[template_name]
+                    if '$ref' in v['items']:  # search for reference somewhere else in the root schema
+                        for i in v['items']['$ref'].split('/'):
+                            if i == '#':
+                                aux = self.container.schema
+                            else:
+                                aux = aux.get(i)
+                        schema = aux
                     else:
-                        schema = v['items'][0]
+                        schema = v['items']
                     for index in range(v['minItems']):
                         iform = SchemaForm(schema=schema, key=f'{k}-{index}', parent_form=self)
                         value.append(iform)
@@ -315,6 +327,9 @@ class SchemaForm(dbc.Card):
                 # If field is an array of strings, e.g. NWBFile.experimenter
                 elif isinstance(v['items'], dict):
                     value = v
+                # Other cases
+                else:
+                    value = v
 
             # If field is a simple input field, e.g. description
             elif 'type' in v and v['type'] in ['string', 'number', 'boolean']:
@@ -354,6 +369,7 @@ class SchemaFormContainer(html.Div):
         self.parent_app = parent_app
         self.data = {}
         self.children_forms = []
+        self.skiped_forms = []
 
         if root_path is not None:
             self.parent_app.server.config['DATA_PATH'] = root_path
@@ -405,6 +421,29 @@ class SchemaFormContainer(html.Div):
             State(dict(_args_dict, data_type='number'), 'value'),
         ]
 
+        self.parent_app.clientside_callback(
+            """
+            function(n_clicks, state){
+
+                 //const element = document.getElementById(JSON.stringify(ids, Object.keys(ids).sort()))
+
+                 ctx = dash_clientside.callback_context
+                 
+                 if (typeof ctx.triggered[0] === "undefined"){
+                     return dash_clientside.no_update
+                 }
+                 if (n_clicks){
+                     return !state
+                 }
+                 return dash_clientside.no_update
+
+            }
+            """,
+            Output({"type":'collapsible-body', "container": f'{self.id}', "index": MATCH}, 'is_open'),
+            [Input({"type": 'collapsible-toggle', "container": f'{self.id}', "index": MATCH}, 'n_clicks')],
+            [State({"type":'collapsible-body', "container": f'{self.id}', "index": MATCH}, 'is_open')]
+        )
+
         @self.parent_app.callback(
             Output(f'{self.id}-output-update-finished-verification', 'children'),
             [Input(f'{self.id}-external-trigger-update-internal-dict', 'children')],
@@ -593,9 +632,11 @@ class SchemaFormContainer(html.Div):
         """Update data in the internal mapping dictionary of this Container"""
         if key is None:
             key = self.id
-
+    
         # Update dict with incoming data
         for k, v in data.items():
+            if k in self.skiped_forms:
+                continue
             # If value is a dictionary
             if isinstance(v, dict):
                 if key != '':
@@ -606,6 +647,9 @@ class SchemaFormContainer(html.Div):
             # If value is a list of dicts
             elif isinstance(v, list) and len(v) > 0 and isinstance(v[0], dict):
                 self.update_lists_data(v, key, k)
+            # If no value on list of dicts
+            elif isinstance(v, list) and len(v) == 0:
+                continue
             # If value is a string, number, list of strings or boolean
             else:
                 component_id = key + '-' + k  # e.g. NWBFile-session_description
@@ -615,11 +659,16 @@ class SchemaFormContainer(html.Div):
         # Construct children forms
         if 'properties' in self.schema:
             for form_key, form_value in self.schema['properties'].items():
+                if "renderForm" in form_value and not form_value['renderForm']:
+                    self.skiped_forms.extend(form_key)
+                    continue
                 iform = SchemaForm(
                     schema=form_value,
                     key=form_key,
                     container=self
                 )
+                if iform.skiped_forms:
+                    self.skiped_forms.extend(iform.skiped_forms)
                 self.children_forms.append(iform)
         self.children = self.children_forms + self.children_triggers
 
